#ifndef PERILOUS_GAMESYS_C
#define PERILOUS_GAMESYS_C

#include <math.h>
#include "compflag.h"
#include "gamesys.h"
#include "gamefunc.h"
#include "hiscore.h"
#include "music.h"

// try to set a gfx mode (emphasis on the word try)
int pp_gfxset(int c) {
  set_color_depth(c);
  return (!set_gfx_mode(pp_GFX_DRIVER, pp_SCREEN_WIDTH, pp_SCREEN_HEIGHT, 0, 0));
}

// this happens every second
void pp_timer_bps1(void) {
  #ifdef pp_SHOWFPS
  char title[256];
  #endif
  pp_globals.fps=pp_globals.fpsc;
  pp_globals.fpsc=0;
  #ifdef pp_SHOWFPS
  // throw the fps into the title
  sprintf(title, "PERILOUS PEREGRINATION by Inphernic for TINS '03 - FPS: %d", pp_globals.fps);
  set_window_title(title);
  #endif
}

// this happens every 1/20 second
void pp_timer_bps20(void) {
  pp_globals.game_bps20=1;
  // right.
}

// this happens every 1/60 second
void pp_timer_bps60(void) {
  // updates the angles used for snazzy IL&M fx
  pp_globals.game_bps60=1;
  pp_globals.fx_angle+=.1;
  if (pp_globals.fx_angle>6.28) pp_globals.fx_angle-=6.28;
  pp_globals.fx_angle2+=.05;
  if (pp_globals.fx_angle2>6.28) pp_globals.fx_angle2-=6.28;
  pp_globals.fx_angle3+=.03;
  if (pp_globals.fx_angle3>6.28) pp_globals.fx_angle3-=6.28;
}

// this is pretty ugly, but it sets one particle
void pp_particle_set(unsigned int i, unsigned char state, float x, float y, float z, float xvel, float yvel, float zvel, int ttl, int colour) {
  if (i>pp_MAX_PARTICLES) return;
  pp_globals.particles[i].x=x;
  pp_globals.particles[i].y=y;
  pp_globals.particles[i].z=z;
  pp_globals.particles[i].xvel=xvel;
  pp_globals.particles[i].yvel=yvel;
  pp_globals.particles[i].zvel=zvel;
  pp_globals.particles[i].ttl=ttl;
  pp_globals.particles[i].maxttl=ttl;
  pp_globals.particles[i].colour=colour;
  pp_globals.particles[i].state=state;
}

// this is a bit nicer, creates one particle
void pp_particle_create(float x, float y, float z, float xvel, float yvel, float zvel, unsigned int ttl, int colour) {
  pp_particle_set(pp_globals.particle_run, 1, x, y, z, xvel, yvel, zvel, ttl, colour);
  pp_globals.particle_run++;
  while (pp_globals.particle_run>=pp_MAX_PARTICLES) pp_globals.particle_run-=pp_MAX_PARTICLES;
}

// update particle logic
void pp_particle_logic(void) {
  int i;
  for (i=0; i<pp_MAX_PARTICLES; i++) {  
    if (pp_globals.particles[i].state) {
      pp_globals.particles[i].x+=pp_globals.particles[i].xvel;
      pp_globals.particles[i].y+=pp_globals.particles[i].yvel;
      pp_globals.particles[i].z+=pp_globals.particles[i].zvel;
      pp_globals.particles[i].ttl--;
      if (pp_globals.particles[i].ttl<=0) pp_globals.particles[i].state=0;
    }
  }
}

// draw the particles in the main menu. i actually intended to use particles
// somewhere else too, but i guess they wouldn't have fitter anywhere
void pp_main_particle_draw(void) {
  int i;
  drawing_mode(DRAW_MODE_TRANS, NULL, 0, 0);
  for (i=0; i<pp_MAX_PARTICLES; i++) {  
    if (pp_globals.particles[i].state) {
      set_add_blender(100,100,100,
        255.0*((float)pp_globals.particles[i].ttl/(float)pp_globals.particles[i].maxttl)
      );
      // the particles rotate - they actually form a 3d infinity symbol in the
      // main menu, but there's so much dispersion it might not be visible unless you
      // know what you are looking
      circlefill(pp_globals.dblbuf, (pp_SCREEN_WIDTH/2)+((int)pp_globals.particles[i].x*cos(pp_globals.fx_angle3))+((int)pp_globals.particles[i].z*sin(pp_globals.fx_angle3)), (pp_SCREEN_HEIGHT/2)+(int)pp_globals.particles[i].y, 2.0*((float)pp_globals.particles[i].ttl/(float)pp_globals.particles[i].maxttl), pp_globals.particles[i].colour);
    }
  }
  drawing_mode(DRAW_MODE_SOLID, NULL, 0, 0);
}

// initialize the game stuffs
int pp_init(void) {
  int temp;
  pp_globals.dblbuf=NULL;
  pp_globals.game_exit=0;
  pp_globals.game_phase=0;
  pp_globals.game_bps20=0;
  pp_globals.particle_run=0;
  pp_globals.fx_angle=0.0;
  pp_globals.fx_angle2=0.0;  
  pp_globals.fx_angle3=0.0;  
  pp_globals.intro_played=0;
  pp_globals.think=NULL;
  allegro_init();
  srand(time(NULL));
  for(temp=0; temp<pp_MAX_PARTICLES; temp++) pp_particle_set(temp, 0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0, 0);
  if (install_timer()) {
    allegro_message("Could not initialize the timers.");
    pp_exit(); return 0;
  }
  if (install_keyboard()) {
    allegro_message("Could not initialize the keyboard.");
    pp_exit(); return 0;
  }
  temp=desktop_color_depth();
  if (!temp) temp=16;
  if (!pp_gfxset(temp)) if (!pp_gfxset(16)) if (!pp_gfxset(32)) {
    allegro_message("Could not set up video mode.");
    pp_exit(); return 0;
  }
  // hahahaa!!1one
  if (set_display_switch_mode(SWITCH_BACKAMNESIA)==-1) {
    if (set_display_switch_mode(SWITCH_BACKGROUND)==-1) {
      set_display_switch_mode(SWITCH_AMNESIA);
    }
  }  
  #ifdef pp_MUSIC
  if (!install_sound(DIGI_AUTODETECT, MIDI_NONE, "")) {
    pp_install_music();
  }
  #endif
  pp_globals.dblbuf=create_bitmap(pp_SCREEN_WIDTH, pp_SCREEN_HEIGHT);
  if (!pp_globals.dblbuf) {
    allegro_message("Not enough memory for screen buffer.");
    pp_exit(); return 0;
  }
  pp_globals.think=NULL;
  pp_globals.think=pp_texgen(1, 32, 32);
  set_window_title("PERILOUS PEREGRINATION by Inphernic for TINS '03");
  pp_globals.sysdat=NULL;
  pp_globals.sysdat=load_datafile("system.dat");
  if (pp_globals.sysdat) {
    sysfont=font;
    font=pp_globals.sysdat[0].dat;
  }
  pp_globals.mouseicon=NULL;
  pp_globals.mouse=0;
  #ifdef pp_MOUSE
  if (install_mouse()>0) {
    pp_globals.mouse=1;
    pp_globals.mouseicon=pp_texgen(7, 32, 32);
  }
  #endif
  install_int_ex(pp_timer_bps1, BPS_TO_TIMER(1));
  install_int_ex(pp_timer_bps20, BPS_TO_TIMER(20));
  install_int_ex(pp_timer_bps60, BPS_TO_TIMER(60));
  pp_hiscore_set_defaults();
  pp_draw_wait();
  pp_hiscore_read();
  pp_hiscore_save();
  text_mode(-1);
  pp_gamedata.bgl=-1;
  pp_gamedata.bg=NULL;
  scrupdl=0;
  #ifdef pp_GHETTOBOOST
  scrupdl=1;
  #endif
  return 1;
}

// remove everything we used and go the sleep
void pp_exit(void) {
  if (pp_globals.mouse) remove_mouse();
  pp_globals.mouse=0;
  #ifdef pp_MUSIC
  pp_remove_music();
  remove_sound();
  #endif
  remove_int(pp_timer_bps60);
  remove_int(pp_timer_bps20);
  if (pp_globals.sysdat) {
    font=sysfont;
    unload_datafile(pp_globals.sysdat);
    pp_globals.sysdat=NULL;
  }
  if (pp_gamedata.bg) destroy_bitmap(pp_gamedata.bg);
  if (pp_globals.dblbuf) destroy_bitmap(pp_globals.dblbuf);
  if (pp_globals.think) destroy_bitmap(pp_globals.think);
  if (pp_globals.mouseicon) destroy_bitmap(pp_globals.mouseicon);
  remove_keyboard();
  remove_timer();
  allegro_exit();
}

// blit the doublebuffer bitmap on the screen
void pp_blit(void) {
  blit(pp_globals.dblbuf, screen, 0, 0, 0, 0, pp_SCREEN_WIDTH, pp_SCREEN_HEIGHT);
  pp_globals.fpsc++;  
  yield_timeslice(); // we don't want to be evil
}

#endif
