#ifndef PERILOUS_GAMESYS_H
#define PERILOUS_GAMESYS_H

#include <allegro.h>
#include "compflag.h"

typedef struct {
  float x, xvel; // x position and velocity
  float y, yvel; // y position and velocity
  float z, zvel; // y position and velocity
  int ttl, maxttl, colour; // time to live, max ttl, particle colour
  unsigned char state; // particle state (0 = dead, !0 = active)
} pp_particle;

typedef struct {
  int res[12]; // result of the matching comparison (the colour code)
  int elm[12]; // the element used
} pp_board_row;

struct {
  BITMAP *dblbuf; // doublebuffer bitmap
  char game_exit; // flag - exit game
  char game_phase; // what phase is the game logic in?
  char game_bps20; // is the 20bps timer triggered?
  char game_bps60; // is the 60bps timer triggered?
  char intro_played; // has the intro been played?
  char mouse; // is mouse initialized?
  pp_particle particles[pp_MAX_PARTICLES]; // particle array
  unsigned int particle_run; // current particle index
  float fx_angle, fx_angle2, fx_angle3; // angle value used for fx
  BITMAP *think; // thinking icon
  BITMAP *mouseicon; // mouse icon
  DATAFILE *sysdat; // system datafile pointer
  int fpsc, fps;  
} pp_globals;

struct {
  int level; // current level
  int score; // current score
  int time, maxtime; // time and maximum time for the level
  BITMAP *bg; // background bitmap
  int bgl, jello; // background bitmap index, jello flag
  int rows, elms; // rows allowed, elements allowed
  pp_board_row leveldata[6]; // level data
  int entry; // number of the entry you are modifying
  int key_level[6]; // the code for "level" (as element indices)
  int key_jello[6]; // the code for "jello" (as element indices)
} pp_gamedata;

FONT *sysfont; // just a temp pointer for allegro's font
int mouseclick; // has mouse been clicked (click flag)
int mousetemp; // don't care about this one
int scrupdl; // is ghettoboost used (boost flag)

// try to set a gfx mode
int pp_gfxset(int c);

// trigger every sec
void pp_timer_bps1(void);

// trigger every 1/20 sec
void pp_timer_bps20(void);

// trigger every 1/60 sec
void pp_timer_bps60(void);

// initialize the game
int pp_init(void);

// exit from the game
void pp_exit(void);

// set a particle
void pp_particle_set(unsigned int i, unsigned char state, float x, float y, float z, float xvel, float yvel, float zvel, int ttl, int colour);

// create a particle
void pp_particle_create(float x, float y, float z, float xvel, float yvel, float zvel, unsigned int ttl, int colour);

// update particle logic
void pp_particle_logic(void);

// draw particles in main menu
void pp_main_particle_draw(void);

// initialize the game
void pp_blit(void);

#endif
